import logging
import uuid
import dbus
import dbus.mainloop.glib

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
logHandler = logging.StreamHandler()
formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
logHandler.setFormatter(formatter)
logger.addHandler(logHandler)

logger.setLevel(logging.DEBUG)

class WifiConnectionManager:
    def __init__(self):
        dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
        self.ssid = ""
        self.psk = ""
        self.bus = dbus.SystemBus()
        self.nm_proxy = self.bus.get_object("org.freedesktop.NetworkManager", "/org/freedesktop/NetworkManager")
        self.nm_interface = dbus.Interface(self.nm_proxy, "org.freedesktop.NetworkManager")

    def connect_to_wifi(self):
        try:
            logger.debug(f"Attempting to connect to {self.ssid}")
            connection_params = {
                "802-11-wireless": {
                    "ssid": dbus.ByteArray(self.ssid.encode("utf-8")),
                },
                "802-11-wireless-security": {
                    "key-mgmt": "wpa-psk",
                    "psk": self.psk,
                },
                "connection": {
                    "id": self.ssid,
                    "type": "802-11-wireless",
                    "uuid": str(uuid.uuid4()),
                },
            }

            settings_path, connection_path = self.nm_interface.AddAndActivateConnection(
                connection_params, self.nm_interface.GetDeviceByIpIface("wlan0"), "/"
            )

            connection_proxy = self.bus.get_object("org.freedesktop.NetworkManager", connection_path)
            connection_properties = dbus.Interface(connection_proxy, "org.freedesktop.DBus.Properties")

            # Wait for the connection to be fully activated
            while True:
                state = connection_properties.Get("org.freedesktop.NetworkManager.Connection.Active", "State")
                if state == 2:  # NM_ACTIVE_CONNECTION_STATE_ACTIVATED
                    logger.info(f"Successfully connected to {self.ssid}")
                    return True
                elif state == 3 or state == 4:  # NM_ACTIVE_CONNECTION_STATE_DEACTIVATED or NM_ACTIVE_CONNECTION_STATE_FAILED
                    logger.info(f"Failed to connect to {self.ssid}")
                    return False

        except dbus.exceptions.DBusException as e:
            logger.info(f"D-Bus error: {e}")
            return False

    def SetSsid(self, ssid):
        self.ssid = ssid

    def SetPsk(self, psk):
        self.psk = psk
